"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_device_detect_1 = require("react-device-detect");
const optimize_client_rects_1 = __importDefault(require("./optimize-client-rects"));
const isClientRectInsidePageRect = (clientRect, pageRect) => {
    if (clientRect.top < pageRect.top) {
        return false;
    }
    if (clientRect.bottom > pageRect.bottom) {
        return false;
    }
    if (clientRect.right > pageRect.right) {
        return false;
    }
    if (clientRect.left < pageRect.left) {
        return false;
    }
    return true;
};
const getRenderingClientRects = (range) => __awaiter(void 0, void 0, void 0, function* () {
    return new Promise((resolve, _reject) => {
        setTimeout(() => {
            resolve(Array.from(range.getClientRects()));
        }, react_device_detect_1.isMobile ? 0 : 400);
    });
});
const getClientRects = (range, pages, shouldOptimize = true, viewport) => __awaiter(void 0, void 0, void 0, function* () {
    const clientRects = yield getRenderingClientRects(range);
    const rects = [];
    for (const clientRect of clientRects) {
        for (const page of pages) {
            const pageRect = page.node.getBoundingClientRect();
            if (isClientRectInsidePageRect(clientRect, pageRect) &&
                clientRect.width > 0 &&
                clientRect.height > 0 &&
                clientRect.width < pageRect.width &&
                clientRect.height < pageRect.height) {
                const bounds = viewport
                    .convertToPdfPoint(clientRect.left - pageRect.x, clientRect.top - pageRect.y)
                    .concat(viewport.convertToPdfPoint(clientRect.right - pageRect.x, clientRect.bottom - pageRect.y));
                const highlightedRect = {
                    top: Math.min(bounds[1], bounds[3]),
                    left: Math.min(bounds[0], bounds[2]),
                    width: Math.abs(bounds[0] - bounds[2]),
                    height: Math.abs(bounds[1] - bounds[3]),
                    pageNumber: page.number
                };
                // previous
                // const highlightedRect = {
                //   top: clientRect.top + page.node.scrollTop - pageRect.top,
                //   left: clientRect.left + page.node.scrollLeft - pageRect.left,
                //   width: clientRect.width,
                //   height: clientRect.height,
                //   pageNumber: page.number
                // } as LTWHP
                rects.push(highlightedRect);
            }
        }
    }
    return shouldOptimize ? (0, optimize_client_rects_1.default)(rects) : rects;
});
exports.default = getClientRects;
//# sourceMappingURL=get-client-rects.js.map